import View from './view';
import Viewport from '../viewports/viewport';
import mat4_lookAt from 'gl-mat4/lookAt';
import mat4_perspective from 'gl-mat4/perspective';
const DEGREES_TO_RADIANS = Math.PI / 180;
export default class PerspectiveView extends View {
  _getViewport(props) {
    const x = props.x,
          y = props.y,
          width = props.width,
          height = props.height,
          viewState = props.viewState;
    const eye = viewState.eye,
          _viewState$lookAt = viewState.lookAt,
          lookAt = _viewState$lookAt === void 0 ? [0, 0, 0] : _viewState$lookAt,
          _viewState$up = viewState.up,
          up = _viewState$up === void 0 ? [0, 1, 0] : _viewState$up; // Projection matrix arguments
    // TODO - Extracting from viewState is deprecated

    const fovy = props.fovy || viewState.fovy || 75; // Field of view covered by camera

    const near = props.near || viewState.near || 1; // Distance of near clipping plane

    const far = props.far || viewState.far || 100; // Distance of far clipping plane

    const aspect = Number.isFinite(viewState.aspect) ? viewState.aspect : width / height;
    const fovyRadians = fovy * DEGREES_TO_RADIANS;
    return new Viewport({
      id: this.id,
      x,
      y,
      width,
      height,
      viewMatrix: mat4_lookAt([], eye, lookAt, up),
      projectionMatrix: mat4_perspective([], fovyRadians, aspect, near, far)
    });
  }

}
PerspectiveView.displayName = 'PerspectiveView';
//# sourceMappingURL=perspective-view.js.map